<?php
namespace nur\sery;

use nur\sery\file\base\FileReader;
use nur\sery\file\base\FileWriter;
use nur\sery\file\base\MemoryStream;
use nur\sery\file\base\SharedFile;
use nur\sery\file\base\TempStream;
use nur\sery\file\base\TmpfileWriter;

/**
 * Class file: outils pour gérer les fichiers
 */
class file {
  static function reader($input, ?callable $func=null): FileReader {
    $file = new FileReader($input);
    if ($func !== null) {
      try {
        $func($file);
      } finally {
        $file->close();
      }
    }
    return $file;
  }
  
  static function writer($output, ?string $mode=null, ?callable $func=null): FileWriter {
    $file = new FileWriter($output, $mode);
    if ($func !== null) {
      try {
        $func($file);
      } finally {
        $file->close();
      }
    }
    return $file;
  }

  static function shared($file, ?callable $func=null): SharedFile {
    $file  = new SharedFile($file);
    if ($func !== null) {
      try {
        $func($file);
      } finally {
        $file ->close();
      }
    }
    return $file;
  }

  static function tmpwriter($destdir=null, ?callable $func=null): TmpfileWriter {
    $file = new TmpfileWriter($destdir);
    if ($func !== null) {
      try {
        $func($file);
      } finally {
        $file->close();
      }
    }
    return $file;
  }

  static function memory(?callable $func=null): MemoryStream {
    $file = new MemoryStream();
    if ($func !== null) {
      try {
        $func($file);
      } finally {
        $file->close();
      }
    }
    return $file;
  }

  static function temp(?callable $func=null): TempStream {
    $file = new TempStream();
    if ($func !== null) {
      try {
        $func($file);
      } finally {
        $file->close();
      }
    }
    return $file;
  }
}
